/* $Revision: 1.3 $ */
// Automatically generated by MATLAB Project Wizard version 1.0


/*	demap_12dpam12_partition_mex.c
 *
 *  Brett McClellan, Solarflare Communications
 *  bmcclellan@solarflare.com
 *
 * 12D PAM12 demapping of the partitions to uncoded bits
 * inputs are 3 x 4 ternary partition values, TA(1:3),TB(1:3),TC(1:3) & TD(1:3)
 *  3 level:  -8,0,+8   or the full PAM12 levels: -11 to 11
 * returns w,x1,x2,y1,y2,z1,z2
 *  which are 1-bit,3-bit,3-bit,3-bit,3-bit,3-bit,3-bit values
 * This is a MEX-file for MATLAB.
 *
 */
/* $ Revision: 1.00 $ */

#include "mex.h"



char demap_2d(char TA_temp, char TB_temp)
{
	char tx_data_group_ab_02 ;
	//Pair AB demapping
	switch(TA_temp)
	{
	case(-8) :
		{
        switch(TB_temp)
		{
        case(-8) :	//000
			{
            tx_data_group_ab_02 = 0;
			break ;
			}
        case(0)	:	//001
			{
            tx_data_group_ab_02 = 1;
			break ;
			}
        case(8)	:	//011
			{
            tx_data_group_ab_02 = 3;
			break ;
			}
        break;
		}
		break;
		}
	case(0) :
		{
        switch(TB_temp)
		{
        case(-8) :	//100
			{
            tx_data_group_ab_02 = 4;
			break ;
			}
        case(0) :	//ESC case
			{
            tx_data_group_ab_02 = 0;
			break ;
			}
         case(8)	:	//010
			{
            tx_data_group_ab_02 = 2;
			break ;
			}
        break;
		}
		break;
		}
	case(8) :
		{
        switch(TB_temp)
		{
        case(-8) :	//101
			{
            tx_data_group_ab_02 = 5;
			break ;
			}
        case(0):	//111
			{
            tx_data_group_ab_02 = 7;
			break ;
			}
        case(8) :	//110
			{
            tx_data_group_ab_02 = 6;
			break ;
			}
        break;
		}
		break;
		}
	}

	return tx_data_group_ab_02 ;
}


int demap_2dpam12_donut(char TA,char TB)
{
	char TA_temp, TB_temp ;
	char tx_data_group_ab_02 ;



	// first decode the points within the partitions
	TA_temp = ((TA+12)%8)-4;
	TB_temp = ((TB+12)%8)-4;

	// now decode the partitions
	TA_temp = TA - TA_temp;
	TB_temp = TB - TB_temp;

	tx_data_group_ab_02 = demap_2d( TA_temp, TB_temp);


	return tx_data_group_ab_02 ;


}

bool ESC_test(char ra,char rb)
{
	bool esc_test;
	esc_test = ( (ra > -4) && (ra < 4) && (rb > -4) && (rb < 4) );
	return esc_test ;
}



void demap_12dpam12_partition(double *ta,double *tb,double *tc,double *td, double *w, double *x1, double *x2, double *y1,double *y2, double *z1, double *z2)
{
	char ra0,rb0,rc0,rd0,ra1,rb1,rc1,rd1,ra2,rb2,rc2,rd2 ;
	int rw; //,rx,ry,rz;
	bool ESC_ab0,ESC_cd0,ESC_ab1,ESC_cd1,ESC_ab2,ESC_cd2 ;

	char rx_data_0_ab_02, rx_data_0_cd_02 ;
	char rx_data_1_ab_02, rx_data_1_cd_02 ;
	char rx_data_2_ab_02, rx_data_2_cd_02 ;

	char ESC_value ;


	ra0 = *ta; rb0 = *tb; rc0 = *tc; rd0 = *td;
	ra1 = *(ta+1); rb1 = *(tb+1); rc1 = *(tc+1); rd1 = *(td+1);
	ra2 = *(ta+2); rb2 = *(tb+2); rc2 = *(tc+2); rd2 = *(td+2);

	// first test for ESC patterns
	ESC_ab0 = ESC_test(ra0,rb0);
	ESC_cd0 = ESC_test(rc0,rd0);
	ESC_ab1 = ESC_test(ra1,rb1);
	ESC_cd1 = ESC_test(rc1,rd1);
	ESC_ab2 = ESC_test(ra2,rb2);
	ESC_cd2 = ESC_test(rc2,rd2);

	rx_data_0_ab_02 = demap_2dpam12_donut((char)*(ta),(char)*(tb));
	rx_data_0_cd_02 = demap_2dpam12_donut((char)*(tc),(char)*(td));
	rx_data_1_ab_02 = demap_2dpam12_donut((char)*(ta+1),(char)*(tb+1));
	rx_data_1_cd_02 = demap_2dpam12_donut((char)*(tc+1),(char)*(td+1));
	rx_data_2_ab_02 = demap_2dpam12_donut((char)*(ta+2),(char)*(tb+2));
	rx_data_2_cd_02 = demap_2dpam12_donut((char)*(tc+2),(char)*(td+2));

	// recover the original 02 bits based on the ESC pattern
	ESC_value = (ESC_ab0<<5) + (ESC_cd0<<4) + (ESC_ab1<<3) + (ESC_cd1<<2) + (ESC_ab2<<1) + (ESC_cd2) ;

	rw = (ESC_value > 0);

	switch (ESC_value)
	{
	case(0) :	// no ESC patterns	x x x x x x
		{
			// no changes

			break;
		}
	case(32) :	//000		ESC x x x x x
		{
			// no changes
			rx_data_0_ab_02 = 0;
			break;
		}
	case(16) :	//001		x ESC x x x x
		{
			//  rx_data_0_ab_02 -> rx_data_0_cd_02
			rx_data_0_cd_02 = rx_data_0_ab_02;
			rx_data_0_ab_02 = 1;
			break;
		}
	case(8) :	//010		x x ESC x x x
		{
			//  rx_data_0_cd_02 -> rx_data_1_ab_02
			//  rx_data_0_ab_02 -> rx_data_0_cd_02
			rx_data_1_ab_02 = rx_data_0_cd_02;
			rx_data_0_cd_02 = rx_data_0_ab_02;
			rx_data_0_ab_02 = 2;
			break;
		}
	case(4) :	//011		x x x ESC x x
		{
			//  rx_data_1_ab_02 -> rx_data_1_cd_02
			//  rx_data_0_cd_02 -> rx_data_1_ab_02
			//  rx_data_0_ab_02 -> rx_data_0_cd_02
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_cd_02;
			rx_data_0_cd_02 = rx_data_0_ab_02;
			rx_data_0_ab_02 = 3;
			break;
		}
	case(2) :	//100		x x x x ESC x
		{
			//  rx_data_1_cd_02 -> rx_data_2_ab_02
			//  rx_data_1_ab_02 -> rx_data_1_cd_02
			//  rx_data_0_cd_02 -> rx_data_1_ab_02
			//  rx_data_0_ab_02 -> rx_data_0_cd_02
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_cd_02;
			rx_data_0_cd_02 = rx_data_0_ab_02;
			rx_data_0_ab_02 = 4;
			break;
		}
	case(1) :	//101		x x x x x ESC
		{
			//  rx_data_2_ab_02 -> rx_data_2_cd_02
			//  rx_data_1_cd_02 -> rx_data_2_ab_02
			//  rx_data_1_ab_02 -> rx_data_1_cd_02
			//  rx_data_0_cd_02 -> rx_data_1_ab_02
			//  rx_data_0_ab_02 -> rx_data_0_cd_02
			rx_data_2_cd_02 = rx_data_2_ab_02;
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_cd_02;
			rx_data_0_cd_02 = rx_data_0_ab_02;
			rx_data_0_ab_02 = 5;
			break;
		}
	case(48) :	//110	000		ESC ESC x x x x
		{
			rx_data_0_cd_02 = 0;
			rx_data_0_ab_02 = 6;
			break;
		}
	case(40) :	//110	001		ESC x ESC x x x
		{
			//   rx_data_1_ab_02 <- rx_data_0_cd_02
			rx_data_1_ab_02 = rx_data_0_cd_02;
			rx_data_0_cd_02 = 1;
			rx_data_0_ab_02 = 6;
			break;
		}
	case(36) :	//110	010		ESC x x ESC x x
		{
			//  rx_data_1_cd_02 <- rx_data_1_ab_02
			//  rx_data_1_ab_02 <- rx_data_0_cd_02
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_cd_02;
			rx_data_0_cd_02 = 2;
			rx_data_0_ab_02 = 6;
			break;
		}
	case(34) :	//110	011		ESC x x x ESC x
		{
			//  rx_data_2_ab_02 <- rx_data_1_cd_02
			//  rx_data_1_cd_02 <- rx_data_1_ab_02
			//  rx_data_1_ab_02 <- rx_data_0_cd_02
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_cd_02;
			rx_data_0_cd_02 = 3;
			rx_data_0_ab_02 = 6;
			break;
		}
	case(33) :	//110	100		ESC x x x x ESC
		{
			//  rx_data_2_cd_02 <- rx_data_2_ab_02
			//  rx_data_2_ab_02 <- rx_data_1_cd_02
			//  rx_data_1_cd_02 <- rx_data_1_ab_02
			//  rx_data_1_ab_02 <- rx_data_0_cd_02
			rx_data_2_cd_02 = rx_data_2_ab_02;
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_cd_02;
			rx_data_0_cd_02 = 4;
			rx_data_0_ab_02 = 6;
			break;
		}
	case(24) :	//110	101		x ESC ESC x x x
		{
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 5;
			rx_data_0_ab_02 = 6;
			break;
		}
	case(20) :	//110	110		x ESC x ESC x x
		{
			//  rx_data_1_cd_02 <- rx_data_1_ab_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 6;
			rx_data_0_ab_02 = 6;
			break;
		}
	case(18) :	//110	111		x ESC x x ESC x
		{
			//  rx_data_2_ab_02 <- rx_data_1_cd_02
			//  rx_data_1_cd_02 <- rx_data_1_ab_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 6;
			break;
		}
	case(17) :	//111	000		x ESC x x x ESC
		{
			//  rx_data_2_cd_02 <- rx_data_2_ab_02
			//  rx_data_2_ab_02 <- rx_data_1_cd_02
			//  rx_data_1_cd_02 <- rx_data_1_ab_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_2_cd_02 = rx_data_2_ab_02;
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 0;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(12) :	//111	001		x x ESC ESC x x
		{
			//  rx_data_1_cd_02 <- rx_data_0_cd_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_1_cd_02 = rx_data_0_cd_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 1;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(10) :	//111	010		x x ESC x ESC x
		{
			//  rx_data_3_ab_02 <- rx_data_1_cd_02
			//  rx_data_1_cd_02 <- rx_data_0_cd_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_0_cd_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 2;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(9) :	//111	011		x x ESC x x ESC
		{
			//  rx_data_3_cd_02 <- rx_data_3_ab_02
			//  rx_data_3_ab_02 <- rx_data_1_cd_02
			//  rx_data_1_cd_02 <- rx_data_0_cd_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_2_cd_02 = rx_data_2_ab_02;
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_0_cd_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 3;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(6) :	//111	100		x x x ESC ESC x
		{
			//  rx_data_2_ab_02 <- rx_data_1_ab_02
			//  rx_data_1_cd_02 <- rx_data_0_cd_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_2_ab_02 = rx_data_1_ab_02;
			rx_data_1_cd_02 = rx_data_0_cd_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 4;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(5) :	//111	101		x x x ESC x ESC
		{
			//  rx_data_2_cd_02 <- rx_data_2_ab_02
			//  rx_data_2_ab_02 <- rx_data_1_ab_02
			//  rx_data_1_cd_02 <- rx_data_0_cd_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_2_cd_02 = rx_data_2_ab_02;
			rx_data_2_ab_02 = rx_data_1_ab_02;
			rx_data_1_cd_02 = rx_data_0_cd_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 5;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(3) :	//111	110		x x x x ESC ESC
		{
			//  rx_data_2_cd_02 <- rx_data_1_cd_02
			//  rx_data_2_ab_02 <- rx_data_1_ab_02
			//  rx_data_1_cd_02 <- rx_data_0_cd_02
			//  rx_data_1_ab_02 <- rx_data_0_ab_02
			rx_data_2_cd_02 = rx_data_1_cd_02;
			rx_data_2_ab_02 = rx_data_1_ab_02;
			rx_data_1_cd_02 = rx_data_0_cd_02;
			rx_data_1_ab_02 = rx_data_0_ab_02;
			rx_data_0_cd_02 = 6;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(56) :	//111	111 000		ESC ESC ESC x x x
		{
			rx_data_1_ab_02 = 0;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(7) :	//111	111 001		 x x x ESC ESC ESC
		{
			//  rx_data_2_cd_02 <- rx_data_1_ab_02
			//  rx_data_2_ab_02 <- rx_data_0_cd_02
			//  rx_data_1_cd_02 <- rx_data_0_ab_02
			rx_data_2_cd_02 = rx_data_1_ab_02;
			rx_data_2_ab_02 = rx_data_0_cd_02;
			rx_data_1_cd_02 = rx_data_0_ab_02;
			rx_data_1_ab_02 = 1;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(50) :	//111	111 010		ESC ESC x x ESC x
		{
			//  rx_data_2_ab_02 <- rx_data_1_cd_02
			//  rx_data_1_cd_02 <- rx_data_1_ab_02
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_1_ab_02;
			rx_data_1_ab_02 = 2;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(13) :	//111	111 011		x x ESC ESC x ESC
		{
			//  rx_data_2_cd_02 <- rx_data_2_ab_02
			//  rx_data_2_ab_02 <- rx_data_0_cd_02
			//  rx_data_1_cd_02 <- rx_data_0_ab_02
			rx_data_2_cd_02 = rx_data_2_ab_02;
			rx_data_2_ab_02 = rx_data_0_cd_02;
			rx_data_1_cd_02 = rx_data_0_ab_02;
			rx_data_1_ab_02 = 3;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(44) :	//111	111 100		ESC x ESC ESC x x
		{
			//  rx_data_1_cd_02 <- rx_data_0_cd_02
			rx_data_1_cd_02 = rx_data_0_cd_02;
			rx_data_1_ab_02 = 4;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(19) :	//111	111 101		x ESC x x ESC ESC
		{
			//  rx_data_2_cd_02 <- rx_data_1_cd_02
			//  rx_data_2_ab_02 <- rx_data_1_ab_02
			//  rx_data_1_cd_02 <- rx_data_0_ab_02
			rx_data_2_cd_02 = rx_data_1_cd_02;
			rx_data_2_ab_02 = rx_data_1_ab_02;
			rx_data_1_cd_02 = rx_data_0_ab_02;
			rx_data_1_ab_02 = 5;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(41) :	//111	111 110		ESC x ESC x x ESC
		{
			//  rx_data_2_cd_02 <- rx_data_2_ab_02
			//  rx_data_2_ab_02 <- rx_data_1_cd_02
			//  rx_data_1_cd_02 <- rx_data_0_cd_02
			rx_data_2_cd_02 = rx_data_2_ab_02;
			rx_data_2_ab_02 = rx_data_1_cd_02;
			rx_data_1_cd_02 = rx_data_0_cd_02;
			rx_data_1_ab_02 = 6;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
			break;
		}
	case(22) :	//111	111 111		x ESC x ESC ESC x
		{
			//  rx_data_2_ab_02 <- rx_data_1_ab_02
			//  rx_data_1_cd_02 <- rx_data_0_ab_02
			rx_data_2_ab_02 = rx_data_1_ab_02;
			rx_data_1_cd_02 = rx_data_0_ab_02;
			rx_data_1_ab_02 = 7;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
			break;
		}
	default :
		{
			mexWarnMsgTxt("Unknown ESC case");
			// Equations below are a default for the unknown cases
			//   there may be a better demapping
			rx_data_2_ab_02 = rx_data_1_ab_02;
			rx_data_1_cd_02 = rx_data_0_ab_02;
			rx_data_1_ab_02 = 7;
			rx_data_0_cd_02 = 7;
			rx_data_0_ab_02 = 7;
		}

	}


	// return the 1-bit and six 3-bit values
	*w = rw;
	*x1 = rx_data_0_ab_02;
	*x2 = rx_data_0_cd_02;
	*y1 = rx_data_1_ab_02;
	*y2 = rx_data_1_cd_02;
	*z1 = rx_data_2_ab_02;
	*z2 = rx_data_2_cd_02;




}



/* The gateway routine */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
//  double *y,*z;
//  double  x;
//  int     status,mrows,ncols;
  double *w,*x1,*x2,*y1,*y2,*z1,*z2 ;
//  double *w,*x,*y,*z ; // for output arguments
  double *ta,*tb,*tc,*td ; // for input arguments

  /*  Check for proper number of arguments. */
  /* NOTE: You do not need an else statement when using
     mexErrMsgTxt within an if statement. It will never
     get to the else statement if mexErrMsgTxt is executed.
     (mexErrMsgTxt breaks you out of the MEX-file.)
  */
  if(nrhs != 4)
    mexErrMsgTxt("Four inputs required.");
  if(nlhs != 7)
    mexErrMsgTxt("Seven outputs required.");

  /* Check to make sure the first input argument is a scalar. */
  if(!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) ||
      mxGetN(prhs[0])*mxGetM(prhs[0]) != 3 ) {
    mexErrMsgTxt("Input TA must be 1x3 vector.");
  }
  if(!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) ||
      mxGetN(prhs[1])*mxGetM(prhs[0]) != 3 ) {
    mexErrMsgTxt("Input TB must be 1x3 vector.");
  }
  if(!mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]) ||
      mxGetN(prhs[2])*mxGetM(prhs[0]) != 3 ) {
    mexErrMsgTxt("Input TC must be 1x3 vector.");
  }
  if(!mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) ||
      mxGetN(prhs[3])*mxGetM(prhs[0]) != 3 ) {
    mexErrMsgTxt("Input TD must be 1x3 vector.");
  }


  /* Get the pointer to the TA input vector. */
  ta =  mxGetPr(prhs[0]);
  tb =  mxGetPr(prhs[1]);
  tc =  mxGetPr(prhs[2]);
  td =  mxGetPr(prhs[3]);

  /* Create a 1-by-1 array for the return arguments. */
   plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
   plhs[1] = mxCreateDoubleMatrix(1,1, mxREAL);
   plhs[2] = mxCreateDoubleMatrix(1,1, mxREAL);
   plhs[3] = mxCreateDoubleMatrix(1,1, mxREAL);
   plhs[4] = mxCreateDoubleMatrix(1,1, mxREAL);
   plhs[5] = mxCreateDoubleMatrix(1,1, mxREAL);
   plhs[6] = mxCreateDoubleMatrix(1,1, mxREAL);

  /* Assign a pointer to the output. */
   w = mxGetPr(plhs[0]);
   x1 = mxGetPr(plhs[1]);
   x2 = mxGetPr(plhs[2]);
   y1 = mxGetPr(plhs[3]);
   y2 = mxGetPr(plhs[4]);
   z1 = mxGetPr(plhs[5]);
   z2 = mxGetPr(plhs[6]);

   /* Call the C subroutine. */
  demap_12dpam12_partition(ta,tb,tc,td, w,x1,x2,y1,y2,z1,z2);

}