#include "mex.h"
/*
 * map_12dpam12_partition_mex.c
 *
 * Brett McClellan, Solarflare Communications
 * bmcclellan@solarflare.com
 *
 * 12D PAM12 mapping of the partition selections only
 * input is w,x1,x2,y1,y2,z1,z2
 *  which are 1-bit,3-bit,3-bit,3-bit,3-bit,3-bit,3-bit values
 * returns 3 x 4 ternary partition values, TA(1:3),TB(1:3),TC(1:3) & TD(1:3)
 *    -8,0,+8
 *
 * This is a MEX-file for MATLAB.
 */
/* $ Revision: 1.00 $ */




void map_2dpam12_partition_donut(double *ta,double *tb,int data_02)
{
	char x1,y1;


  //Pair AB mapping
  switch(data_02)
  {
    case 0:     //000
        {x1=-1; y1=-1;break;}
    case 1:     //001
        {x1=-1; y1=0;break;}
    case 3:     //011
        {x1=-1; y1=1;break;}
    case 2:     //010
        {x1=0; y1=1;break;}
    case 6:     //110
        {x1=1; y1=1;break;}
    case 7:     //111
        {x1=1; y1=0;break;}
    case 5:     //101
        {x1=1; y1=-1;break;}
    case 4:     //100
        {x1=0; y1=-1;break;}
  }

  *ta = x1*8;
  *tb = y1*8;


}





void map_12dpam12_partition(double *ta,double *tb,double *tc,double *td, int w, int x1, int x2, int y1,int y2, int z1, int z2)
{

  // w is the select bit

  // if w = 0, use standard mappings
  if (!w)
  {
	  map_2dpam12_partition_donut(ta  ,tb  , x1) ; // Pair A&B, first symbol
	  map_2dpam12_partition_donut(tc  ,td  , x2) ; // Pair C&D, first symbol
	  map_2dpam12_partition_donut(ta+1,tb+1, y1) ; // Pair A&B, second symbol
	  map_2dpam12_partition_donut(tc+1,td+1, y2) ; // Pair C&D, second symbol
	  map_2dpam12_partition_donut(ta+2,tb+2, z1) ; // Pair A&B, third symbol
	  map_2dpam12_partition_donut(tc+2,td+2, z2) ; // Pair C&D, third symbol

  }
  else
  {
	  // set initial values to zero
	  *ta = 0; *tb = 0;
	  *(ta+1) = 0; *(tb+1) = 0;
	  *(ta+2) = 0; *(tb+2) = 0;

	  //ESC symbol is the donut hole, use first 3 bits of x to select ESC pattern
	  switch (x1)
	  {
		  case 0:     //000		ESC x x x x x
			  {

//				  map_2dpam12_esc  (ta,  tb);
				  map_2dpam12_partition_donut(tc,  td,  x2);
				  map_2dpam12_partition_donut(ta+1,tb+1,y1);
				  map_2dpam12_partition_donut(tc+1,td+1,y2);
				  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  map_2dpam12_partition_donut(tc+2,td+2,z2);
				  break;
			  }

		  case 1:     //001		x ESC x x x x
			  {
				  map_2dpam12_partition_donut(ta,  tb,  x2);
//				  map_2dpam12_esc  (tc,  td);
				  map_2dpam12_partition_donut(ta+1,tb+1,y1);
				  map_2dpam12_partition_donut(tc+1,td+1,y2);
				  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  map_2dpam12_partition_donut(tc+2,td+2,z2);
				  break;
			  }
		  case 2:     //010		x x ESC x x x
			  {
				  map_2dpam12_partition_donut(ta,  tb,  x2);
				  map_2dpam12_partition_donut(tc,  td,  y1);
//				  map_2dpam12_esc  (ta+1,tb+1);
				  map_2dpam12_partition_donut(tc+1,td+1,y2);
				  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  map_2dpam12_partition_donut(tc+2,td+2,z2);
				  break;
			  }
		  case 3:     //011		x x x ESC x x
			  {
				  map_2dpam12_partition_donut(ta,  tb,  x2);
				  map_2dpam12_partition_donut(tc,  td,  y1);
				  map_2dpam12_partition_donut(ta+1,tb+1,y2);
//				  map_2dpam12_esc  (tc+1,td+1);
				  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  map_2dpam12_partition_donut(tc+2,td+2,z2);
				  break;
			  }
		  case 4:     //100		x x x x ESC x
			  {
				  map_2dpam12_partition_donut(ta,  tb,  x2);
				  map_2dpam12_partition_donut(tc,  td,  y1);
				  map_2dpam12_partition_donut(ta+1,tb+1,y2);
				  map_2dpam12_partition_donut(tc+1,td+1,z1);
//				  map_2dpam12_esc  (ta+2,tb+2);
				  map_2dpam12_partition_donut(tc+2,td+2,z2);
				  break;
			  }

		  case 5:     //101		x x x x x ESC
			  {
				  map_2dpam12_partition_donut(ta,  tb,  x2);
				  map_2dpam12_partition_donut(tc,  td,  y1);
				  map_2dpam12_partition_donut(ta+1,tb+1,y2);
				  map_2dpam12_partition_donut(tc+1,td+1,z1);
				  map_2dpam12_partition_donut(ta+2,tb+2,z2);
//				  map_2dpam12_esc  (tc+2,td+2);
				  break;
			  }

		  case 6:     //110
			  {
				  switch (x2)
				  {
					  case 0:     //110	000		ESC ESC x x x x
						  {
//							  map_2dpam12_esc  (ta,  tb);
//							  map_2dpam12_esc  (tc,  td);
							  map_2dpam12_partition_donut(ta+1,tb+1,y1);
				  			  map_2dpam12_partition_donut(tc+1,td+1,y2);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 1:     //110	001		ESC x ESC x x x
						  {
//							  map_2dpam12_esc  (ta,  tb);
							  map_2dpam12_partition_donut(tc,  td,  y1);
//							  map_2dpam12_esc  (ta+1,tb+1);
				  			  map_2dpam12_partition_donut(tc+1,td+1,y2);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 2:     //110	010		ESC x x ESC x x
						  {
//							  map_2dpam12_esc  (ta,  tb);
							  map_2dpam12_partition_donut(tc,  td,  y1);
							  map_2dpam12_partition_donut(ta+1,tb+1,y2);
//				  			  map_2dpam12_esc  (tc+1,td+1);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 3:     //110	011		ESC x x x ESC x
						  {
//							  map_2dpam12_esc  (ta,  tb);
							  map_2dpam12_partition_donut(tc,  td,  y1);
							  map_2dpam12_partition_donut(ta+1,tb+1,y2);
				  			  map_2dpam12_partition_donut(tc+1,td+1,z1);
//				  			  map_2dpam12_esc  (ta+2,tb+2);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 4:     //110	100		ESC x x x x ESC
						  {
//							  map_2dpam12_esc  (ta,  tb);
							  map_2dpam12_partition_donut(tc,  td,  y1);
							  map_2dpam12_partition_donut(ta+1,tb+1,y2);
				  			  map_2dpam12_partition_donut(tc+1,td+1,z1);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z2);
//				  			  map_2dpam12_esc  (tc+2,td+2);
							  break;
						  }
					  case 5:     //110	101		x ESC ESC x x x
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
//							  map_2dpam12_esc  (tc,  td);
//							  map_2dpam12_esc  (ta+1,tb+1);
				  			  map_2dpam12_partition_donut(tc+1,td+1,y2);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 6:     //110	110		x ESC x ESC x x
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
//							  map_2dpam12_esc  (tc,  td);
							  map_2dpam12_partition_donut(ta+1,tb+1,y2);
//				  			  map_2dpam12_esc  (tc+1,td+1);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 7:     //110	111		x ESC x x ESC x
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
//							  map_2dpam12_esc  (tc,  td);
							  map_2dpam12_partition_donut(ta+1,tb+1,y2);
				  			  map_2dpam12_partition_donut(tc+1,td+1,z1);
//				  			  map_2dpam12_esc  (ta+2,tb+2);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }

					  default :
						  {mexErrMsgTxt("Unknown ESC case 6:."); }

				  }

				  break;
			  }

		  case 7:     //111
			  {
				  switch (x2)
				  {
					  case 0:     //111	000		x ESC x x x ESC
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
//							  map_2dpam12_esc  (tc,  td);
							  map_2dpam12_partition_donut(ta+1,tb+1,y2);
				  			  map_2dpam12_partition_donut(tc+1,td+1,z1);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z2);
//				  			  map_2dpam12_esc  (tc+2,td+2);
							  break;
						  }
					  case 1:     //111	001		x x ESC ESC x x
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
							  map_2dpam12_partition_donut(tc,  td,  y2);
//							  map_2dpam12_esc  (ta+1,tb+1);
//				  			  map_2dpam12_esc  (tc+1,td+1);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 2:     //111	010		x x ESC x ESC x
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
							  map_2dpam12_partition_donut(tc,  td,  y2);
//							  map_2dpam12_esc  (ta+1,tb+1);
				  			  map_2dpam12_partition_donut(tc+1,td+1,z1);
//				  			  map_2dpam12_esc  (ta+2,tb+2);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 3:     //111	011		x x ESC x x ESC
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
							  map_2dpam12_partition_donut(tc,  td,  y2);
//							  map_2dpam12_esc  (ta+1,tb+1);
				  			  map_2dpam12_partition_donut(tc+1,td+1,z1);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z2);
//				  			  map_2dpam12_esc  (tc+2,td+2);
							  break;
						  }
					  case 4:     //111	100		x x x ESC ESC x
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
							  map_2dpam12_partition_donut(tc,  td,  y2);
							  map_2dpam12_partition_donut(ta+1,tb+1,z1);
//				  			  map_2dpam12_esc  (tc+1,td+1);
//				  			  map_2dpam12_esc  (ta+2,tb+2);
				  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
							  break;
						  }
					  case 5:     //111	101		x x x ESC x ESC
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
							  map_2dpam12_partition_donut(tc,  td,  y2);
							  map_2dpam12_partition_donut(ta+1,tb+1,z1);
//				  			  map_2dpam12_esc  (tc+1,td+1);
				  			  map_2dpam12_partition_donut(ta+2,tb+2,z2);
//				  			  map_2dpam12_esc  (tc+2,td+2);
							  break;
						  }

					  case 6:     //111	110		x x x x ESC ESC
						  {
							  map_2dpam12_partition_donut(ta,  tb,  y1);
							  map_2dpam12_partition_donut(tc,  td,  y2);
							  map_2dpam12_partition_donut(ta+1,tb+1,z1);
				  			  map_2dpam12_partition_donut(tc+1,td+1,z2);
//				  			  map_2dpam12_esc  (ta+2,tb+2);
//				  			  map_2dpam12_esc  (tc+2,td+2);
							  break;
						  }
					  case 7:     //111	111		ESC ESC ESC x x x
						  {
							  switch (y1)
							  {
								  case 0:     //111	111 000		ESC ESC ESC x x x
									  {
//										  map_2dpam12_esc  (ta,  tb);
//							  			  map_2dpam12_esc  (tc,  td);
//							  			  map_2dpam12_esc  (ta+1,tb+1);
				  			  			  map_2dpam12_partition_donut(tc+1,td+1,y2);
				  			  			  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  			  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
										  break;
									  }
								  case 1:     //111	111 001		 x x x ESC ESC ESC
									  {
										  map_2dpam12_partition_donut(ta,  tb,  y2);
							  			  map_2dpam12_partition_donut(tc,  td,  z1);
							  			  map_2dpam12_partition_donut(ta+1,tb+1,z2);
//				  			  			  map_2dpam12_esc  (tc+1,td+1);
//				  			  			  map_2dpam12_esc  (ta+2,tb+2);
//				  			  			  map_2dpam12_esc  (tc+2,td+2);
										  break;
									  }
								  case 2:     //111	111 010		ESC ESC x x ESC x
									  {
//										  map_2dpam12_esc  (ta,  tb);
//							  			  map_2dpam12_esc  (tc,  td);
							  			  map_2dpam12_partition_donut(ta+1,tb+1,y2);
				  			  			  map_2dpam12_partition_donut(tc+1,td+1,z1);
//				  			  			  map_2dpam12_esc  (ta+2,tb+2);
				  			  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
										  break;
									  }
								  case 3:     //111	111 011		x x ESC ESC x ESC
									  {
										  map_2dpam12_partition_donut(ta,  tb,  y2);
							  			  map_2dpam12_partition_donut(tc,  td,  z1);
//							  			  map_2dpam12_esc  (ta+1,tb+1);
//				  			  			  map_2dpam12_esc  (tc+1,td+1);
				  			  			  map_2dpam12_partition_donut(ta+2,tb+2,z2);
//				  			  			  map_2dpam12_esc  (tc+2,td+2);
										  break;
									  }
								  case 4:     //111	111 100		ESC x ESC ESC x x
									  {
//										  map_2dpam12_esc  (ta,  tb);
							  			  map_2dpam12_partition_donut(tc,  td,  y2);
//							  			  map_2dpam12_esc  (ta+1,tb+1);
//				  			  			  map_2dpam12_esc  (tc+1,td+1);
				  			  			  map_2dpam12_partition_donut(ta+2,tb+2,z1);
				  			  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
										  break;
									  }
								  case 5:     //111	111 101		x ESC x x ESC ESC
									  {
										  map_2dpam12_partition_donut(ta,  tb,  y2);
//							  			  map_2dpam12_esc  (tc,  td);
							  			  map_2dpam12_partition_donut(ta+1,tb+1,z1);
				  			  			  map_2dpam12_partition_donut(tc+1,td+1,z2);
//				  			  			  map_2dpam12_esc  (ta+2,tb+2);
//				  			  			  map_2dpam12_esc  (tc+2,td+2);
										  break;
									  }
								  case 6:     //111	111 110		ESC x ESC x x ESC
									  {
//										  map_2dpam12_esc  (ta,  tb);
							  			  map_2dpam12_partition_donut(tc,  td,  y2);
//							  			  map_2dpam12_esc  (ta+1,tb+1);
				  			  			  map_2dpam12_partition_donut(tc+1,td+1,z1);
				  			  			  map_2dpam12_partition_donut(ta+2,tb+2,z2);
//				  			  			  map_2dpam12_esc  (tc+2,td+2);
										  break;
									  }
								  case 7:     //111	111 111		x ESC x ESC ESC x
									  {
										  map_2dpam12_partition_donut(ta,  tb,  y2);
//							  			  map_2dpam12_esc  (tc,  td);
							  			  map_2dpam12_partition_donut(ta+1,tb+1,z1);
//				  			  			  map_2dpam12_esc  (tc+1,td+1);
//				  			  			  map_2dpam12_esc  (ta+2,tb+2);
				  			  			  map_2dpam12_partition_donut(tc+2,td+2,z2);
										  break;
									  }
								  default :
									  {mexErrMsgTxt("Unknown ESC case 7:7."); }

							  }

							  break;
						  }


					  default :
						  {mexErrMsgTxt("Unknown ESC case 7:."); }

				  }

				  break;
			  }





		  default :
			  {
				  mexErrMsgTxt("Unknown ESC case: x1.");
			  }


	  }



  }





}
/* The gateway routine */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
{
//  double *y,*z;
//  double  x;
  int     status,mrows,ncols;
  unsigned int w,x1,x2,y1,y2,z1,z2 ;
  double *ta,*tb,*tc,*td ;

  /*  Check for proper number of arguments. */
  /* NOTE: You do not need an else statement when using
     mexErrMsgTxt within an if statement. It will never
     get to the else statement if mexErrMsgTxt is executed.
     (mexErrMsgTxt breaks you out of the MEX-file.)
  */
  if(nrhs != 7)
    mexErrMsgTxt("Seven inputs required.");
  if(nlhs != 4)
    mexErrMsgTxt("Four outputs required.");

  /* Check to make sure the first input argument is a scalar. */
  if(!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) ||
      mxGetN(prhs[0])*mxGetM(prhs[0]) != 1 ) {
    mexErrMsgTxt("Input w must be a scalar.");
  }
  if(!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1]) ||
      mxGetN(prhs[1])*mxGetM(prhs[0]) != 1 ) {
    mexErrMsgTxt("Input x1 must be a scalar.");
  }
  if(!mxIsDouble(prhs[2]) || mxIsComplex(prhs[2]) ||
      mxGetN(prhs[2])*mxGetM(prhs[0]) != 1 ) {
    mexErrMsgTxt("Input x2 must be a scalar.");
  }
  if(!mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) ||
      mxGetN(prhs[3])*mxGetM(prhs[0]) != 1 ) {
    mexErrMsgTxt("Input y1 must be a scalar.");
  }
  if(!mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) ||
      mxGetN(prhs[3])*mxGetM(prhs[0]) != 1 ) {
    mexErrMsgTxt("Input y2 must be a scalar.");
  }
  if(!mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) ||
      mxGetN(prhs[3])*mxGetM(prhs[0]) != 1 ) {
    mexErrMsgTxt("Input z1 must be a scalar.");
  }
  if(!mxIsDouble(prhs[3]) || mxIsComplex(prhs[3]) ||
      mxGetN(prhs[3])*mxGetM(prhs[0]) != 1 ) {
    mexErrMsgTxt("Input z2 must be a scalar.");
  }

  /* Get the scalar input x. */
  w = mxGetScalar(prhs[0]);
  /* Get the scalar input x. */
  x1 = mxGetScalar(prhs[1]);
  /* Get the scalar input x. */
  x2 = mxGetScalar(prhs[2]);
  /* Get the scalar input x. */
  y1 = mxGetScalar(prhs[3]);
 /* Get the scalar input x. */
  y2 = mxGetScalar(prhs[4]);
  /* Get the scalar input x. */
  z1 = mxGetScalar(prhs[5]);
  /* Get the scalar input x. */
  z2 = mxGetScalar(prhs[6]);

  /* Create a pointer to the input matrix y. */
  //y = mxGetPr(prhs[1]);

  /* Get the dimensions of the matrix input y. */
  //mrows = mxGetM(prhs[1]);
  //ncols = mxGetN(prhs[1]);

  /* Set the output pointer to the output matrix. */
  //plhs[0] = mxCreateDoubleMatrix(mrows,ncols, mxREAL);
  /* Create a 1-by-3 array of signed 8-bit integers. */
  //plhs[0] = mxCreateNumericArray(1,1, mxINT8_CLASS,mxREAL);
  //plhs[1] = mxCreateNumericArray(1,1, mxINT8_CLASS,mxREAL);
  //plhs[2] = mxCreateNumericArray(1,1, mxINT8_CLASS,mxREAL);
  //plhs[3] = mxCreateNumericArray(1,1, mxINT8_CLASS,mxREAL);
   plhs[0] = mxCreateDoubleMatrix(1,3, mxREAL);
   plhs[1] = mxCreateDoubleMatrix(1,3, mxREAL);
   plhs[2] = mxCreateDoubleMatrix(1,3, mxREAL);
   plhs[3] = mxCreateDoubleMatrix(1,3, mxREAL);

  /* Create a C pointer to a copy of the output matrix. */
  ta = mxGetPr(plhs[0]);
  tb = mxGetPr(plhs[1]);
  tc = mxGetPr(plhs[2]);
  td = mxGetPr(plhs[3]);
  /* Call the C subroutine. */
  //xtimesy(x,y,z,mrows,ncols);
  map_12dpam12_partition(ta,tb,tc,td, w,x1,x2,y1,y2,z1,z2);
}