
% Example mapping of a single frame for 12D 12PAM
% Brett McClellan, Solarflare Communications
% bmcclellan@solarflare.com

clear x x1 x2 y1 y2 z1 z2 TX_4d12pam

framesize = 126;
len_uncoded = framesize/3*19; 
len_coded = framesize*8;


% Generate uncoded bits
uncoded = (sign(randn(1,len_uncoded))+1)/2;
% Generate coded bits
coded = (sign(randn(1,len_coded))+1)/2; % note the parity bits are random just for test purposes

% Example framing, break uncoded into 19-bit words
% w is 1-bit, x1,x2,y1,y2,z1,z2 are 3-bit
for ii=1:(framesize/3)    %3*42 = 126 baud  
    w(ii) = uncoded( (ii-1)*19+(1))' ;   
    x1(ii) = [ 4 2 1] * uncoded( (ii-1)*19+(2:4))' ;
    x2(ii) = [ 4 2 1] * uncoded( (ii-1)*19+(5:7))' ;
    y1(ii) = [ 4 2 1] * uncoded( (ii-1)*19+(8:10))' ;
    y2(ii) = [ 4 2 1] * uncoded( (ii-1)*19+(11:13))' ;
    z1(ii) = [ 4 2 1] * uncoded( (ii-1)*19+(14:16))' ;
    z2(ii) = [ 4 2 1] * uncoded( (ii-1)*19+(17:19))' ;    
    
    % map 19-bit uncoded words into 12D partitions
    [TA,TB,TC,TD] = map_12dpam12_partition_mex(w(ii),x1(ii),x2(ii),y1(ii),y2(ii),z1(ii),z2(ii)); 
    TX_4d12pam((ii-1)*3+(1:3),1) = TA';
    TX_4d12pam((ii-1)*3+(1:3),2) = TB';
    TX_4d12pam((ii-1)*3+(1:3),3) = TC';
    TX_4d12pam((ii-1)*3+(1:3),4) = TD';

end
% map the coded bits into the 1D cosets, 4 pair x framesize
tx_coded = map_4dcoded(coded,framesize) ;
% add the cosets to the partitions
TX_4d12pam = TX_4d12pam + tx_coded ;

